#!/usr/bin/env node
'use strict';

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _meow = require('meow');

var _meow2 = _interopRequireDefault(_meow);

var _filesize = require('filesize');

var _filesize2 = _interopRequireDefault(_filesize);

var _cliTable = require('cli-table');

var _cliTable2 = _interopRequireDefault(_cliTable);

var _lodash = require('lodash');

var _ = _interopRequireWildcard(_lodash);

var _links = require('./links');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var cli = (0, _meow2.default)('\n    Usage\n      $ weigh <input>\n\n    Options\n      --path, -P  Path to starting file/folder\n      --depth, -d  How deep should the table show\n\n    Examples\n      $ weigh --path src/index.html\n');

var flags = cli.flags;

var sourcePath = flags.path ? _path2.default.resolve(flags.path) : process.cwd();
var tDepth = flags.depth;
var store = {};
var table = new _cliTable2.default({
  head: ['#', 'Type', 'Link', 'Net Weight', 'Standalone Weight', 'file size'],
  colWidths: [5, 10, 80, 15, 20, 15],
  style: { head: ['red'], border: ['white'] },
  chars: { 'mid': '', 'left-mid': '', 'mid-mid': '', 'right-mid': '' }
});

try {
  (function () {
    var stat = _fs2.default.statSync(sourcePath);
    var isDir = stat.isDirectory();
    var indexPath = isDir ? _path2.default.resolve(sourcePath, './index.html') : sourcePath;
    (0, _links.parseFile)(indexPath, store).then(function (r) {
      var localLinks = _.filter(_.values(store), function (ast) {
        return ast.urlObject.local;
      });
      var indexWeight = calcOtherWeights(localLinks, indexPath, stat.size);
      var fSize = (0, _filesize2.default)(stat.size, { base: 10 });
      var nSize = (0, _filesize2.default)(indexWeight.nSize, { base: 10 });
      var astColl = _.values(store);
      var visibleColl = void 0;

      if (tDepth) {
        var visibleLinks = [indexPath];
        for (var i = 0; i < tDepth; i += 1) {
          visibleLinks = visibleLinks.concat(walkLinks(astColl, visibleLinks));
        }
        visibleColl = _.values(_.pick.apply(null, [store].concat(visibleLinks)));
      }

      var group = _.groupBy(visibleColl || astColl, function (ast) {
        return ast.urlObject.local ? 'local' : 'external';
      });

      var tableArray = toTableArray(group, stat.size, indexPath);

      var sortedTable = _.sortBy(tableArray, [function (a) {
        return a.detail.size;
      }]);

      sortedTable.push({
        arr: ['Total', _path2.default.basename(indexPath), nSize, ' ⬅ ', fSize],
        detail: { size: nSize }
      });

      _.each(sortedTable.reverse(), function (a, i) {
        a.arr.splice(0, 0, i || '');
        table.push(a.arr);
      });

      console.log(table.toString());
    });
  })();
} catch (e) {
  console.log(e);
}

function walkLinks(links, filePaths) {

  var immChildren = _.filter(links, function (link) {
    return _.find(link.occurances, function (o) {
      return filePaths.indexOf(o.filePath) !== -1;
    });
  });

  return _.map(immChildren, function (link) {
    return link.urlObject.absPath;
  });
}

function calcOtherWeights(links, filePath, fSize) {
  var nSize = 0;
  var sSize = 0;
  var immChildren = _.filter(links, function (link) {
    return _.find(link.occurances, function (o) {
      return o.filePath === filePath;
    });
  });
  if (immChildren) {
    _.each(immChildren, function (c) {
      var childSize = calcOtherWeights(links, c.urlObject.absPath, c.fileStats.size);
      nSize += childSize.sSize;
      if (c.occurances.length === 1) {
        sSize += childSize.sSize;
      }
    });
  }
  nSize += fSize;
  sSize += fSize;
  if (store[filePath]) {
    store[filePath].fileStats.nSize = nSize;
    store[filePath].fileStats.sSize = sSize;
  }
  return { nSize: nSize, sSize: sSize };
}
function toTableArray(group, initialIndexSize, indexPath) {
  var netIndexSize = initialIndexSize;
  var arr = [];
  _.each(group.local, function (ast) {
    netIndexSize += ast.fileStats.size;
    var fSize = (0, _filesize2.default)(ast.fileStats.size, { base: 10 });
    var nSize = (0, _filesize2.default)(ast.fileStats.nSize, { base: 10 });
    var sSize = (0, _filesize2.default)(ast.fileStats.sSize, { base: 10 });
    var absPath = ast.urlObject.absPath;
    var relPath = _path2.default.relative(_path2.default.dirname(indexPath), absPath);
    arr.push({
      arr: [ast.tagName, relPath, nSize, sSize, fSize],
      detail: { size: ast.fileStats.sSize }
    });
  });
  _.each(group.external, function (ast) {
    var absPath = ast.urlObject.absPath;
    arr.push({
      arr: [ast.tagName, absPath, 'NA', 'NA', 'NA'],
      detail: { size: 0 }
    });
  });
  return arr;
}